@@ -1,11 +1,12 @@
 ;;; vc-hooks.el --- resident support for version-control
 
-;; Copyright (C) 1992,93,94,95,96,98,99,2000  Free Software Foundation, Inc.
+;; Copyright (C) 1992,93,94,95,96,98,99,2000,03,2004
+;;           Free Software Foundation, Inc.
 
 ;; Author:     FSF (see vc.el for full credits)
 ;; Maintainer: Andre Spiegel <spiegel@gnu.org>
 
-;; $Id: vc-hooks.el,v 1.134.4.2 2001/12/14 08:19:06 spiegel Exp $
+;; $Id: vc-hooks.el,v 1.161 2004/03/15 03:53:05 monnier Exp $
 
 ;; This file is part of GNU Emacs.
 
@@ -38,11 +39,21 @@
 
 ;; Customization Variables (the rest is in vc.el)
 
-(defvar vc-ignore-vc-files nil "Obsolete -- use `vc-handled-backends'.")
-(defvar vc-master-templates () "Obsolete -- use vc-BACKEND-master-templates.")
-(defvar vc-header-alist () "Obsolete -- use vc-BACKEND-header.")
+(defvar vc-ignore-vc-files nil)
+(make-obsolete-variable 'vc-ignore-vc-files
+                        "set `vc-handled-backends' to nil to disable VC.")
 
-(defcustom vc-handled-backends '(RCS CVS SCCS)
+(defvar vc-master-templates ())
+(make-obsolete-variable 'vc-master-templates 
+ "to define master templates for a given BACKEND, use 
+vc-BACKEND-master-templates.  To enable or disable VC for a given
+BACKEND, use `vc-handled-backends'.")
+
+(defvar vc-header-alist ())
+(make-obsolete-variable 'vc-header-alist 'vc-BACKEND-header)
+
+(defcustom vc-handled-backends '(RCS CVS SVN SCCS Arch MCVS)
+  ;; Arch and MCVS come last because they are per-tree rather than per-dir.
   "*List of version control backends for which VC will be used.
 Entries in this list will be tried in order to determine whether a
 file is under that sort of version control.
@@ -65,7 +76,8 @@ An empty list disables VC altogether."
   "*If non-nil, backups of registered files are made as with other files.
 If nil (the default), files covered by version control don't get backups."
   :type 'boolean
-  :group 'vc)
+  :group 'vc
+  :group 'backup)
 
 (defcustom vc-follow-symlinks 'ask
   "*What to do if visiting a symbolic link to a file under version control.
@@ -114,8 +126,58 @@ See also variable `vc-consult-headers'."
 	   (funcall vc-mistrust-permissions
 		    (vc-backend-subdirectory-name file)))))
 
+(defcustom vc-stay-local t
+  "*Non-nil means use local operations when possible for remote repositories.
+This avoids slow queries over the network and instead uses heuristics
+and past information to determine the current status of a file.
+
+The value can also be a regular expression or list of regular
+expressions to match against the host name of a repository; then VC
+only stays local for hosts that match it.  Alternatively, the value
+can be a list of regular expressions where the first element is the
+symbol `except'; then VC always stays local except for hosts matched
+by these regular expressions."
+  :type '(choice (const :tag "Always stay local" t)
+	  (const :tag "Don't stay local" nil)
+	  (list :format "\nExamine hostname and %v" :tag "Examine hostname ..."
+		(set :format "%v" :inline t (const :format "%t" :tag "don't" except))
+		(regexp :format " stay local,\n%t: %v" :tag "if it matches")
+		(repeat :format "%v%i\n" :inline t (regexp :tag "or"))))
+  :version "21.4"
+  :group 'vc)
+
+(defun vc-stay-local-p (file)
+  "Return non-nil if VC should stay local when handling FILE.
+This uses the `repository-hostname' backend operation."
+  (let* ((backend (vc-backend file))
+	 (sym (vc-make-backend-sym backend 'stay-local))
+	 (stay-local (if (boundp sym) (symbol-value sym) t)))
+    (if (eq stay-local t) (setq stay-local vc-stay-local))
+    (if (symbolp stay-local) stay-local
+      (let ((dirname (if (file-directory-p file)
+			 (directory-file-name file)
+		       (file-name-directory file))))
+	(eq 'yes
+	    (or (vc-file-getprop dirname 'vc-stay-local-p)
+		(vc-file-setprop
+		 dirname 'vc-stay-local-p
+		 (let ((hostname (vc-call-backend
+				  backend 'repository-hostname dirname)))
+		   (if (not hostname)
+		       'no
+		     (let ((default t))
+		       (if (eq (car-safe stay-local) 'except)
+			   (setq default nil stay-local (cdr stay-local)))
+		       (when (consp stay-local)
+			 (setq stay-local
+			       (mapconcat 'identity stay-local "\\|")))
+		       (if (if (string-match stay-local hostname)
+			       default (not default))
+			   'yes 'no)))))))))))
+
+;;; This is handled specially now.
 ;; Tell Emacs about this new kind of minor mode
-(add-to-list 'minor-mode-alist '(vc-mode vc-mode))
+;; (add-to-list 'minor-mode-alist '(vc-mode vc-mode))
 
 (make-variable-buffer-local 'vc-mode)
 (put 'vc-mode 'permanent-local t)
@@ -169,7 +231,7 @@ VC commands are globally reachable under the prefix `\\[vc-prefix-map]':
 
 (defun vc-find-backend-function (backend fun)
   "Return BACKEND-specific implementation of FUN.
-If there is no such implementation, return the default implementation; 
+If there is no such implementation, return the default implementation;
 if that doesn't exist either, return nil."
   (let ((f (vc-make-backend-sym backend fun)))
     (if (fboundp f) f
@@ -191,14 +253,15 @@ and else calls
     (apply 'vc-default-FUN BACKEND ARGS)
 
 It is usually called via the `vc-call' macro."
-  (let ((f (cdr (assoc function-name (get backend 'vc-functions)))))
-    (unless f
+  (let ((f (assoc function-name (get backend 'vc-functions))))
+    (if f (setq f (cdr f))
       (setq f (vc-find-backend-function backend function-name))
-      (put backend 'vc-functions (cons (cons function-name f)
-				       (get backend 'vc-functions))))
-    (if (consp f)
-	(apply (car f) (cdr f) args)
-      (apply f args))))
+      (push (cons function-name f) (get backend 'vc-functions)))
+    (cond
+     ((null f)
+      (error "Sorry, %s is not implemented for %s" function-name backend))
+     ((consp f)	(apply (car f) (cdr f) args))
+     (t		(apply f args)))))
 
 (defmacro vc-call (fun file &rest args)
   ;; BEWARE!! `file' is evaluated twice!!
@@ -216,8 +279,9 @@ It is usually called via the `vc-call' macro."
 
 Optional argument LIMIT is a regexp.  If present, the file is inserted
 in chunks of size BLOCKSIZE (default 8 kByte), until the first
-occurrence of LIMIT is found.  The function returns non-nil if FILE 
-exists and its contents were successfully inserted."
+occurrence of LIMIT is found.  Anything from the start of that occurrence
+to the end of the buffer is then deleted.  The function returns
+non-nil if FILE exists and its contents were successfully inserted."
   (erase-buffer)
   (when (file-exists-p file)
     (if (not limit)
@@ -228,7 +292,10 @@ exists and its contents were successfully inserted."
 	    (and (< 0 (cadr (insert-file-contents
 			     file nil filepos (incf filepos blocksize))))
 		 (progn (beginning-of-line)
-			(not (re-search-forward limit nil 'move)))))))
+                        (let ((pos (re-search-forward limit nil 'move)))
+                          (if pos (delete-region (match-beginning 0)
+                                                 (point-max)))
+                          (not pos)))))))
     (set-buffer-modified-p nil)
     t))
 
@@ -263,7 +330,7 @@ backend is tried first."
 	     (and (vc-call-backend b 'registered file)
 		  (vc-file-setprop file 'vc-backend b)
 		  (throw 'found t)))
-	   (if (or (not backend) (eq backend 'none)) 
+	   (if (or (not backend) (eq backend 'none))
 	       vc-handled-backends
 	     (cons backend vc-handled-backends))))
         ;; File is not registered.
@@ -273,7 +340,7 @@ backend is tried first."
 (defun vc-backend (file)
   "Return the version control type of FILE, nil if it is not registered."
   ;; `file' can be nil in several places (typically due to the use of
-  ;; code like (vc-backend (buffer-file-name))).
+  ;; code like (vc-backend buffer-file-name)).
   (when (stringp file)
     (let ((property (vc-file-getprop file 'vc-backend)))
       ;; Note that internally, Emacs remembers unregistered
@@ -311,14 +378,12 @@ For registered files, the possible values are:
               when the user saves the first changes to the file.
 
   'locking    FILE is read-only if up-to-date; user must type
-              \\[vc-toggle-read-only] before editing.  Strict locking
+              \\[vc-next-action] before editing.  Strict locking
               is assumed.
 
   'announce   FILE is read-only if up-to-date; user must type
-              \\[vc-toggle-read-only] before editing.  But other users
-              may be editing at the same time.
-
-If FILE is not registered, this function always returns nil."
+              \\[vc-next-action] before editing.  But other users
+              may be editing at the same time."
   (or (vc-file-getprop file 'vc-checkout-model)
       (if (vc-backend file)
           (vc-file-setprop file 'vc-checkout-model
@@ -348,7 +413,7 @@ For registered files, the value returned is one of:
 
   USER               The current version of the working file is locked by
                      some other USER (a string).
-            
+
   'needs-patch       The file has not been edited by the user, but there is
                      a more recent version on the current branch stored
                      in the master file.
@@ -364,6 +429,12 @@ For registered files, the value returned is one of:
                      with locking; it represents an erroneous condition that
                      should be resolved by the user (vc-next-action will
                      prompt the user to do it)."
+  ;; FIXME: New (sub)states needed (?):
+  ;; - `added' (i.e. `edited' but with no base version yet,
+  ;;            typically represented by vc-workfile-version = "0")
+  ;; - `conflict' (i.e. `edited' with conflict markers)
+  ;; - `removed'
+  ;; - `copied' and `moved' (might be handled by `removed' and `added')
   (or (vc-file-getprop file 'vc-state)
       (if (vc-backend file)
           (vc-file-setprop file 'vc-state
@@ -379,6 +450,22 @@ It simply calls the real state computation function `vc-BACKEND-state'
 and does not employ any heuristic at all."
    (vc-call-backend backend 'state file))
 
+(defun vc-workfile-unchanged-p (file)
+  "Return non-nil if FILE has not changed since the last checkout."
+  (let ((checkout-time (vc-file-getprop file 'vc-checkout-time))
+        (lastmod (nth 5 (file-attributes file))))
+    (if checkout-time
+        (equal checkout-time lastmod)
+      (let ((unchanged (vc-call workfile-unchanged-p file)))
+        (vc-file-setprop file 'vc-checkout-time (if unchanged lastmod 0))
+        unchanged))))
+
+(defun vc-default-workfile-unchanged-p (backend file)
+  "Check if FILE is unchanged by diffing against the master version.
+Return non-nil if FILE is unchanged."
+  ;; If rev1 is nil, `diff' uses the current workfile version.
+  (zerop (vc-call diff file)))
+
 (defun vc-workfile-version (file)
   "Return the version level of the current workfile FILE.
 If FILE is not registered, this function always returns nil."
@@ -387,8 +474,6 @@ If FILE is not registered, this function always returns nil."
           (vc-file-setprop file 'vc-workfile-version
                            (vc-call workfile-version file)))))
 
-;;; actual version-control code starts here
-
 (defun vc-default-registered (backend file)
   "Check if FILE is registered in BACKEND using vc-BACKEND-master-templates."
   (let ((sym (vc-make-backend-sym backend 'master-templates)))
@@ -400,7 +485,8 @@ If FILE is not registered, this function always returns nil."
 				(and (consp template)
 				     (eq (cdr template) backend)
 				     (car template)))
-			      vc-master-templates))
+                              (with-no-warnings
+                               vc-master-templates)))
 		       (symbol-value sym))))
     (let ((result (vc-check-master-templates file (symbol-value sym))))
       (if (stringp result)
@@ -455,19 +541,22 @@ this function."
 
 (defun vc-toggle-read-only (&optional verbose)
   "Change read-only status of current buffer, perhaps via version control.
+
 If the buffer is visiting a file registered with version control,
 then check the file in or out.  Otherwise, just change the read-only flag
 of the buffer.
 With prefix argument, ask for version number to check in or check out.
 Check-out of a specified version number does not lock the file;
-to do that, use this command a second time with no argument."
+to do that, use this command a second time with no argument.
+
+If you bind this function to \\[toggle-read-only], then Emacs checks files
+in or out whenever you toggle the read-only flag."
   (interactive "P")
   (if (or (and (boundp 'vc-dired-mode) vc-dired-mode)
 	  ;; use boundp because vc.el might not be loaded
-	  (vc-backend (buffer-file-name)))
+	  (vc-backend buffer-file-name))
       (vc-next-action verbose)
     (toggle-read-only)))
-(define-key global-map "\C-x\C-q" 'vc-toggle-read-only)
 
 (defun vc-default-make-version-backups-p (backend file)
   "Return non-nil if unmodified versions should be backed up locally.
@@ -482,8 +571,8 @@ a regexp for matching all such backup files, regardless of the version."
   (if regexp
       (concat (regexp-quote (file-name-nondirectory file))
               "\\.~[0-9.]+" (unless manual "\\.") "~")
-    (expand-file-name (concat (file-name-nondirectory file) 
-                              ".~" (or rev (vc-workfile-version file)) 
+    (expand-file-name (concat (file-name-nondirectory file)
+                              ".~" (or rev (vc-workfile-version file))
                               (unless manual ".") "~")
                       (file-name-directory file))))
 
@@ -501,7 +590,7 @@ a regexp for matching all such backup files, regardless of the version."
   "Make a backup copy of FILE, which is assumed in sync with the repository.
 Before doing that, check if there are any old backups and get rid of them."
   (unless (and (fboundp 'msdos-long-file-names)
-               (not (msdos-long-file-names)))
+               (not (with-no-warnings (msdos-long-file-names))))
     (vc-delete-automatic-version-backups file)
     (copy-file file (vc-version-backup-file-name file)
                nil 'keep-date)))
@@ -511,7 +600,7 @@ Before doing that, check if there are any old backups and get rid of them."
   ;; If the file on disk is still in sync with the repository,
   ;; and version backups should be made, copy the file to
   ;; another name.  This enables local diffs and local reverting.
-  (let ((file (buffer-file-name)))
+  (let ((file buffer-file-name))
     (and (vc-backend file)
 	 (vc-up-to-date-p file)
 	 (eq (vc-checkout-model file) 'implicit)
@@ -523,7 +612,7 @@ Before doing that, check if there are any old backups and get rid of them."
   ;; If the file in the current buffer is under version control,
   ;; up-to-date, and locking is not used for the file, set
   ;; the state to 'edited and redisplay the mode line.
-  (let ((file (buffer-file-name)))
+  (let ((file buffer-file-name))
     (and (vc-backend file)
 	 (or (and (equal (vc-file-getprop file 'vc-checkout-time)
 			 (nth 5 (file-attributes file)))
@@ -546,27 +635,29 @@ Before doing that, check if there are any old backups and get rid of them."
 The value is set in the current buffer, which should be the buffer
 visiting FILE."
   (interactive (list buffer-file-name))
-  (unless (not (vc-backend file))
-    (setq vc-mode (concat " " (if vc-display-status
-				  (vc-call mode-line-string file)
-				(symbol-name (vc-backend file)))))
-    ;; If the file is locked by some other user, make
-    ;; the buffer read-only.  Like this, even root
-    ;; cannot modify a file that someone else has locked.
-    (and (equal file (buffer-file-name))
-         (stringp (vc-state file))
-	 (setq buffer-read-only t))
-    ;; If the user is root, and the file is not owner-writable,
-    ;; then pretend that we can't write it
-    ;; even though we can (because root can write anything).
+  (let ((backend (vc-backend file)))
+    (if (not backend)
+	(setq vc-mode nil)
+      (setq vc-mode (concat " " (if vc-display-status
+				    (vc-call mode-line-string file)
+				  (symbol-name backend))))
+      ;; If the file is locked by some other user, make
+      ;; the buffer read-only.  Like this, even root
+      ;; cannot modify a file that someone else has locked.
+      (and (equal file buffer-file-name)
+	   (stringp (vc-state file))
+	   (setq buffer-read-only t))
+      ;; If the user is root, and the file is not owner-writable,
+      ;; then pretend that we can't write it
+      ;; even though we can (because root can write anything).
     ;; This way, even root cannot modify a file that isn't locked.
-    (and (equal file (buffer-file-name))
+    (and (equal file buffer-file-name)
 	 (not buffer-read-only)
-	 (zerop (user-real-uid))
-	 (zerop (logand (file-modes (buffer-file-name)) 128))
-	 (setq buffer-read-only t)))
-  (force-mode-line-update)
-  (vc-backend file))
+	   (zerop (user-real-uid))
+	   (zerop (logand (file-modes buffer-file-name) 128))
+	   (setq buffer-read-only t)))
+    (force-mode-line-update)
+    backend))
 
 (defun vc-default-mode-line-string (backend file)
   "Return string for placement in modeline by `vc-mode-line' for FILE.
@@ -608,55 +699,62 @@ current, and kill the buffer that visits the link."
       (set-buffer true-buffer)
       (kill-buffer this-buffer))))
 
+(defun vc-default-find-file-hook (backend)
+  nil)
+
 (defun vc-find-file-hook ()
-  "Function for `find-file-hooks' activating VC mode if appropriate."
+  "Function for `find-file-hook' activating VC mode if appropriate."
   ;; Recompute whether file is version controlled,
   ;; if user has killed the buffer and revisited.
+  (if vc-mode
+      (setq vc-mode nil))
   (when buffer-file-name
     (vc-file-clearprops buffer-file-name)
     (cond
      ((vc-backend buffer-file-name)
+      ;; Compute the state and put it in the modeline.
       (vc-mode-line buffer-file-name)
-      (cond ((not vc-make-backup-files)
-	     ;; Use this variable, not make-backup-files,
-	     ;; because this is for things that depend on the file name.
-	     (make-local-variable 'backup-inhibited)
-	     (setq backup-inhibited t))))
+      (unless vc-make-backup-files
+	;; Use this variable, not make-backup-files,
+	;; because this is for things that depend on the file name.
+	(set (make-local-variable 'backup-inhibited) t))
+      ;; Let the backend setup any buffer-local things he needs.
+      (vc-call-backend (vc-backend buffer-file-name) 'find-file-hook))
      ((let* ((link (file-symlink-p buffer-file-name))
 	     (link-type (and link (vc-backend (file-chase-links link)))))
-	(if link-type
-            (cond ((eq vc-follow-symlinks nil)
-                   (message
+	(cond ((not link-type) nil)	;Nothing to do.
+	      ((eq vc-follow-symlinks nil)
+	       (message
         "Warning: symbolic link to %s-controlled source file" link-type))
-                  ((or (not (eq vc-follow-symlinks 'ask))
-		       ;; If we already visited this file by following
-		       ;; the link, don't ask again if we try to visit
-		       ;; it again.  GUD does that, and repeated questions
-		       ;; are painful.
-		       (get-file-buffer
-			(abbreviate-file-name
-                         (file-chase-links buffer-file-name))))
-		       
-		   (vc-follow-link)
-		   (message "Followed link to %s" buffer-file-name)
-		   (vc-find-file-hook))
-                  (t
-                   (if (yes-or-no-p (format
-        "Symbolic link to %s-controlled source file; follow link? " link-type))
-                       (progn (vc-follow-link)
-                              (message "Followed link to %s" buffer-file-name)
-                              (vc-find-file-hook))
-                     (message
-        "Warning: editing through the link bypasses version control")
-                     )))))))))
+	      ((or (not (eq vc-follow-symlinks 'ask))
+		   ;; If we already visited this file by following
+		   ;; the link, don't ask again if we try to visit
+		   ;; it again.  GUD does that, and repeated questions
+		   ;; are painful.
+		   (get-file-buffer
+		    (abbreviate-file-name
+		     (file-chase-links buffer-file-name))))
 
-(add-hook 'find-file-hooks 'vc-find-file-hook)
+	       (vc-follow-link)
+	       (message "Followed link to %s" buffer-file-name)
+	       (vc-find-file-hook))
+	      (t
+	       (if (yes-or-no-p (format
+        "Symbolic link to %s-controlled source file; follow link? " link-type))
+		   (progn (vc-follow-link)
+			  (message "Followed link to %s" buffer-file-name)
+			  (vc-find-file-hook))
+		 (message
+        "Warning: editing through the link bypasses version control")
+		 ))))))))
+
+(add-hook 'find-file-hook 'vc-find-file-hook)
 
 ;; more hooks, this time for file-not-found
 (defun vc-file-not-found-hook ()
   "When file is not found, try to check it out from version control.
 Returns t if checkout was successful, nil otherwise.
-Used in `find-file-not-found-hooks'."
+Used in `find-file-not-found-functions'."
   ;; When a file does not exist, ignore cached info about it
   ;; from a previous visit.
   (vc-file-clearprops buffer-file-name)
@@ -669,15 +767,14 @@ Used in `find-file-not-found-hooks'."
       (setq default-directory (file-name-directory buffer-file-name))
       (not (vc-error-occurred (vc-checkout buffer-file-name))))))
 
-(add-hook 'find-file-not-found-hooks 'vc-file-not-found-hook)
+(add-hook 'find-file-not-found-functions 'vc-file-not-found-hook)
 
 (defun vc-kill-buffer-hook ()
   "Discard VC info about a file when we kill its buffer."
-  (if (buffer-file-name)
-      (vc-file-clearprops (buffer-file-name))))
+  (if buffer-file-name
+      (vc-file-clearprops buffer-file-name)))
 
-;; ??? DL: why is this not done?
-;;;(add-hook 'kill-buffer-hook 'vc-kill-buffer-hook)
+(add-hook 'kill-buffer-hook 'vc-kill-buffer-hook)
 
 ;; Now arrange for (autoloaded) bindings of the main package.
 ;; Bindings for this have to go in the global map, as we'll often
@@ -723,16 +820,18 @@ Used in `find-file-not-found-hooks'."
   (define-key vc-menu-map [vc-rename-file] '("Rename File" . vc-rename-file))
   (define-key vc-menu-map [vc-version-other-window]
     '("Show Other Version" . vc-version-other-window))
-  (define-key vc-menu-map [vc-diff] '("Compare with Last Version" . vc-diff))
+  (define-key vc-menu-map [vc-diff] '("Compare with Base Version" . vc-diff))
   (define-key vc-menu-map [vc-update-change-log]
     '("Update ChangeLog" . vc-update-change-log))
   (define-key vc-menu-map [vc-print-log] '("Show History" . vc-print-log))
   (define-key vc-menu-map [separator2] '("----"))
-  (define-key vc-menu-map [undo] '("Undo Last Check-In" . vc-cancel-version))
-  (define-key vc-menu-map [vc-revert-buffer]
-    '("Revert to Last Version" . vc-revert-buffer))
   (define-key vc-menu-map [vc-insert-header]
     '("Insert Header" . vc-insert-headers))
+  (define-key vc-menu-map [undo] '("Undo Last Check-In" . vc-cancel-version))
+  (define-key vc-menu-map [vc-revert-buffer]
+    '("Revert to Base Version" . vc-revert-buffer))
+  (define-key vc-menu-map [vc-update]
+    '("Update to Latest Version" . vc-update))
   (define-key vc-menu-map [vc-next-action] '("Check In/Out" . vc-next-action))
   (define-key vc-menu-map [vc-register] '("Register" . vc-register)))
 
@@ -755,4 +854,5 @@ Used in `find-file-not-found-hooks'."
 
 (provide 'vc-hooks)
 
+;;; arch-tag: 2e5a6fa7-1d30-48e2-8bd0-e3d335f04f32
 ;;; vc-hooks.el ends here
